#ifdef _USE_WXWIDGETS_
#ifndef __bbcreaMaracasVisuShowNPoints_h_INCLUDED__
#define __bbcreaMaracasVisuShowNPoints_h_INCLUDED__
#include "bbtkWxBlackBox.h"


#include "vtkActor.h"
#include "vtkSphereSource.h"
#include "vtkImageData.h"
#include "vtkRenderer.h"
#include "vtkTextActor3D.h"

namespace bbcreaMaracasVisu
{

  class ShowNPoints;

  //--------------------------------------------------------------------------
  class WidgetShowNPoints : public wxPanel
  {
  public:
	  WidgetShowNPoints( wxWindow *parent,  ShowNPoints *box);
	  ~WidgetShowNPoints(); 
	  void OnAddPoint(wxCommandEvent &event);   
	  void OnRenamePoint(wxCommandEvent& event);
	  void OnErasePoint(wxCommandEvent& event);
	  void OnEraseLastPoint(wxCommandEvent &event);   
	  void OnDeleteAllPoints(wxCommandEvent &event);   
	  void OnSavePoints(wxCommandEvent &event);   
	  void OnLoadPoints(wxCommandEvent &event);

	  void UpdatePoints(wxCommandEvent &event);
	  
	  void SetPoint(std::vector<int> ppoint);
	  void SetColour(std::vector<double> colour);
	  void SetOpacity(double opacity);
	  void SetRadio(double radio);
	  void SetImage(vtkImageData *image);
	  void SetRenderer(vtkRenderer *renderer);
	  std::vector<int> GetLstPointsX();
	  std::vector<int> GetLstPointsY();
	  std::vector<int> GetLstPointsZ();
	  std::vector<std::string> GetLstLabels();

  private:
	  
	  void			AddPoint(int x, int y, int z, std::string label);
	  std::string	CleanSpaces(std::string ss);
	  int			GetNearestPoint();
	  void			ErasePoint(int id);
	  void			SetOutputBox();
	  
	  ShowNPoints					*mbbShowNPoints;
	  vtkRenderer					*renderer;
	  std::vector<int>				lstPointsX;
	  std::vector<int>				lstPointsY;
	  std::vector<int>				lstPointsZ;
	  std::vector<std::string>		lstLabels;
	  std::vector<vtkActor*>		lstActorsSphere;//NTU changed from prop3D to Actor
	  std::vector<vtkTextActor3D*>	lstActorsText;

	  //NTU: For updating points

	  std::vector<vtkSphereSource*> lstSourceSphere;

	  std::vector<int>				mpoint;
	  vtkImageData					*mimage;
	  std::vector<double>			mcolour;
	  double						mopacity;
	  double						mradio;
	  wxTextCtrl					*textCtrl;
	  wxStaticText					*txtNrPoints;
	  wxSlider						*sdrOpacity;
	  wxSlider						*sdrRadio;
  };



class /*BBTK_EXPORT*/ ShowNPoints
 : 
   public bbtk::WxBlackBox
{
  BBTK_BLACK_BOX_INTERFACE(ShowNPoints,bbtk::WxBlackBox);
  BBTK_DECLARE_INPUT(In, std::vector<int> );
  BBTK_DECLARE_INPUT(Renderer, vtkRenderer* );
  BBTK_DECLARE_INPUT(Image, vtkImageData* );
  BBTK_DECLARE_INPUT(Colour, std::vector<double> );
  BBTK_DECLARE_INPUT(Opacity, double );
  BBTK_DECLARE_INPUT(Radio, double );
  BBTK_DECLARE_OUTPUT( lstPointsX, std::vector<int> );
  BBTK_DECLARE_OUTPUT( lstPointsY, std::vector<int> );
  BBTK_DECLARE_OUTPUT( lstPointsZ, std::vector<int> );
  BBTK_DECLARE_OUTPUT( lstLabels,  std::vector<std::string> );
  BBTK_PROCESS(Process);
  void Process();
  BBTK_CREATE_WIDGET(CreateWidget);
  void CreateWidget(wxWindow*);

private:
	WidgetShowNPoints *mwxwidget; 

};

BBTK_BEGIN_DESCRIBE_BLACK_BOX(ShowNPoints,bbtk::WxBlackBox);
  BBTK_NAME("ShowNPoints");
  BBTK_AUTHOR("Eduardo DAVILA");
  BBTK_DESCRIPTION("widget that shows N moints in 3D (vtkActors)");
  BBTK_CATEGORY("widgetVtk");
  BBTK_INPUT(ShowNPoints,In,"One Point",std::vector<int>,"");
  BBTK_INPUT(ShowNPoints,Renderer,"Renderer",vtkRenderer*,"");
  BBTK_INPUT(ShowNPoints,Image,"vktkImageData",vtkImageData*,"");
  BBTK_INPUT(ShowNPoints,Colour,"Colour of the actor",std::vector<double>,"colour");
  BBTK_INPUT(ShowNPoints,Opacity,"Opacity of the actor",double,"");
  BBTK_INPUT(ShowNPoints,Radio,"Radio of the spheres",double,"");
  BBTK_OUTPUT(ShowNPoints , lstPointsX , " list of points X ", std::vector<int> ,"");
  BBTK_OUTPUT(ShowNPoints , lstPointsY , " list of points Y ", std::vector<int> ,"");
  BBTK_OUTPUT(ShowNPoints , lstPointsZ , " list of points Z ", std::vector<int> ,"");
  BBTK_OUTPUT(ShowNPoints , lstLabels , " list of labels ", std::vector<std::string> ,"");
BBTK_END_DESCRIBE_BLACK_BOX(ShowNPoints);
}
// EO namespace bbcreaMaracasVisu

#endif // __bbcreaMaracasVisuShowNPoints_h_INCLUDED__
#endif // _USE_WXWIDGETS_

