/*=========================================================================

  Program:   wxMaracas
  Module:    $RCSfile: CutModelData.h,v $
  Language:  C++
  Date:      $Date: 2009/09/08 08:14:27 $
  Version:   $Revision: 1.3 $

  Copyright: (c) 2002, 2003
  License:
  
     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/




#ifndef __CutModelDataH__
#define __CutModelDataH__

#include <iostream>
#include <vector>

#include "vtkImageData.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkCommand.h"
#include "vtkBoxWidget.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#include "vtkProperty.h"
#include "CutModelException.h"
#include "CutModelFigure.h"
#include "CutModelFigureCube.h"
#include "CutModelFigureSphere.h"
#include "CutModelFigureCylinder.h"

#include <wx/utils.h>

class CutModelData  {

public:
	CutModelData();
	CutModelData(int id, vtkRenderWindowInteractor* interactor, vtkCommand* observer, vtkImageData* img);
	~CutModelData();
	void initializeData(int id, vtkRenderWindowInteractor* interactor, vtkCommand* observer, vtkImageData* img);


	int getId(){
		return _id;
	}

	void changeOpacity(int opacity)throw( CutModelException);

	void ShowViewBox(bool check)throw( CutModelException);

	void ChangeShape(int selection)throw( CutModelException);

	vtkActor* getActor()throw( CutModelException);

	vtkPolyData* getPolyData()throw( CutModelException);
	
	void changeColor(double r,double g,double b)throw( CutModelException);

	void udapteActorDirection()throw( CutModelException);

	void ExecuteCut( double* range, bool isinside, vtkImageData* copyimage)throw( CutModelException);

	void ExecuteUnCut(  bool isinside, vtkImageData* image, vtkImageData* copyimage)throw( CutModelException);

	void RefreshViewBox();

	vtkTransform* getCurrentMatrix(){
		return currentmatrix;
	}

	int getCurrentShape(){
		return _currentshape;
	}

	void setCurrentShape(int currentshape){
		_currentshape = currentshape;
	}

	void setTransform(vtkTransform* transform,vtkImageData* img)throw( CutModelException);
private:	
	
	void checkInvariant()throw( CutModelException);

	void createBoxWidget(vtkRenderWindowInteractor* interactor, vtkCommand* observer);
	void createActor();
	void setTransform(vtkImageData* img)throw( CutModelException);
	void createShapes();	

	vtkBoxWidget* _boxWidgetVolume;
	vtkPolyDataMapper* _Mapper;	
	vtkActor* _Actor;	
	int _id;

	int _currentshape;

	CutModelFigureCube* _cubefigure;
	CutModelFigureCylinder* _cylinderfigure;
	CutModelFigureSphere* _spherefigure;
	vtkTransform* currentmatrix;
	vtkTransform* modeltransform;
	vtkTransform* inversModel;


	CutModelFigure* getCurentCuttingModel();

	vtkTransform* getModelTransform(vtkImageData* copyimage);
	vtkTransform* getModelTransformInvers();
};

#endif 
