#ifndef __ContourPropagation_h_INCLUDED__
#define __ContourPropagation_h_INCLUDED__

#include "Propagation.h"
#include "ExtractControlPoints2D.h"
#include "vtkImageData.h"
#include "vectorFunctions.h"
#include "marTypes.h"

class creaMaracasVisu_EXPORT ContourPropagation 
{
	public:
		ContourPropagation();
		~ContourPropagation();
		void resetAppend				( );
		void appendContour				( std::vector<double>*InX, 
										  std::vector<double>*InY, 
										  std::vector<double>*InZ );
		void GetContour					( int id, std::vector<double>*vecX, 
												  std::vector<double>*vecY, 
												  std::vector<double>*vecZ );
		//AUTOMATIC METHOD
		void GetInitialControlPoints	( int id, std::vector<double>*vecX, 
												  std::vector<double>*vecY, 
												  std::vector<double>*vecZ);
		void GetControlPoints			( int id, std::vector<double>*vecX, 
												  std::vector<double>*vecY, 
												  std::vector<double>*vecZ );

		//SAMPLING METHOD
		void CalculeSplinePropagation();
		void GetControlPoints			( int id, double samp,
										  std::vector<double>*vecX, 
										  std::vector<double>*vecY, 
										  std::vector<double>*vecZ );

		int  FindIdWithZ				( double z);


		void getMaxMinZ					( double *minz, double *maxz);
		void setInterpolationNumber		( int num );


	//TEST CLASSES FOR BBTK
		void appendContour				( );
		vtkImageData* GetKeyContours	( std::vector<double>*vecX, 
										  std::vector<double>*vecY, 
										  std::vector<double>*vecZ, 
										  std::vector<int>*vecS );

		bool ifSliceKeyContourExist(int slice);


	private:
		bool mustOrderAppend( );
		void orderAppend( );
		void OrdenarSliceKeyContourVec();
		

		PropContour					*_propc;
		AutoControlPoints			*_autoc;
		ExtractControlPoints2D		*_ext2D;

		std::vector<double>			_InX;
		std::vector<double>			_InY;
		std::vector<double>			_InZ;
		std::vector<int>			_InS;
		std::vector<int>			_sliceKeyContourVec;

		std::vector<Vector>			_planevector;
};

#endif //__ContourPropagation_h_INCLUDED__
 
 
