/*=========================================================================

  Program:   wxMaracas
  Module:    $RCSfile: wxMaracasSurfaceRendering.h,v $
  Language:  C++
  Date:      $Date: 2010/01/22 17:00:52 $
  Version:   $Revision: 1.5 $

  Copyright: (c) 2002, 2003
  License:

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef __wxMaracasSurfaceRenderingPanelH__
#define __wxMaracasSurfaceRenderingPanelH__

#include <vector>
#include <wx/wx.h>
#include "wx/aui/aui.h"
#include <iostream>
#include "marTypes.h"
#include "vtkProp3D.h"

#include "wxMaracasSurfaceRenderingManager.h"
#include "wxMaracasSurfaceRenderingPanel.h"

class  creaMaracasVisu_EXPORT  wxMaracasSurfaceRendering : public wxPanel
{

public:
    wxMaracasSurfaceRendering( wxWindow* parent, std::string path);
    ~wxMaracasSurfaceRendering( );

    static wxMaracasSurfaceRendering* getInstance(wxWindow* parent,std::string path="");

    static wxMaracasSurfaceRendering* getInstance();

    void setRenderer(vtkRenderer*  renderer);

    void setInteractor(vtkRenderWindowInteractor*  interactor);

    void changeOpacity(int propid, int value);

    void changeIsoValue(int propid, double value);

    void addRemoveActor(int propid, bool addremove);

    void addRemoveSurfaceBox(int propid, bool addremove);

    void changeColor(int propid, double red, double green, double blue);

    void addProp3D(vtkProp3D* prop3D, std::string dataname="");

    void addPropMHD(vtkImageData* imgdata, std::string dataname="");

    void loadProp3D(wxString filename, wxString dataname);

    void loadPropMHD(wxString filename, wxString dataname);

    void onLoadImageFile();

    void addSurfaceRenderingPanel(wxMaracasSurfaceRenderingPanel* surrend, std::string dataname="");

    std::string getPath();

    void deleteActor(int propid);

    bool interactorSet();

private:
    static wxMaracasSurfaceRendering* instance;

    wxMaracasSurfaceRenderingManager* surrendmanager;
	
    wxAuiManager* wxauimanager;
    
    wxToolBar* _toolb;

    std::string _path;

    void createFileChooser();
};

class ToolBarEventHandler : public wxEvtHandler{

	public:
		ToolBarEventHandler();
		~ToolBarEventHandler();

		void onLoadImageFile(wxCommandEvent& event);

	private:

		DECLARE_EVENT_TABLE()
	};

class ToolBar : public wxToolBar{

public:
	ToolBar(wxWindow * parent,std::string iconsdir);
	~ToolBar(void);

private:

	ToolBarEventHandler* _evthand;
};

#endif
