
#include "vtkSTLExtractor.h"
#include <vtkImageGaussianSmooth.h>
#include <vtkMarchingCubes.h>
#include <vtkCleanPolyData.h>
#include <vtkTriangleFilter.h>
#include <vtkImageContinuousDilate3D.h>


vtkSTLExtractor::vtkSTLExtractor(){
	innerSurface = vtkPolyData::New();
	outerSurface = vtkPolyData::New();
	marchingCubesLevel = 128.0;
	sigmaLevel = 100.0;
}

vtkSTLExtractor::~vtkSTLExtractor(){
	innerSurface->Delete();
	outerSurface->Delete();
}

void vtkSTLExtractor::setVolume(vtkImageData *volume){
	this->volume = volume;
}


void vtkSTLExtractor::setMarchingCubesLevel(double level){
	this->marchingCubesLevel = level;
}

void vtkSTLExtractor::setSigmaLevel(double level){
	this->sigmaLevel = level;
}


double vtkSTLExtractor::getMarchingCubesLevel(){
	return marchingCubesLevel;
}

double vtkSTLExtractor::getSigmaLevel(){
	return sigmaLevel;
}


vtkPolyData* vtkSTLExtractor::getInnerSurface(){
	return innerSurface;
}


vtkPolyData* vtkSTLExtractor::getOuterSurface(){
	return outerSurface;
}



void vtkSTLExtractor::calculate(){

	// ------------------------------------------------------------------------
		//  1. CALCULATING THE INNER SURFACE
		// ------------------------------------------------------------------------
	    vtkImageGaussianSmooth *gaussFilter = vtkImageGaussianSmooth::New();
        gaussFilter->SetInput(volume);
        gaussFilter->SetDimensionality(3);
    	gaussFilter->SetStandardDeviation(sigmaLevel);
    
        vtkMarchingCubes *cubesFilter = vtkMarchingCubes::New();
        cubesFilter->SetInput(gaussFilter->GetOutput());
        cubesFilter->SetValue(0,marchingCubesLevel);
		cubesFilter->ComputeGradientsOn ();
        cubesFilter->ComputeScalarsOn ();
		cubesFilter->SetNumberOfContours( 1 );
	    
		// Unir puntos duplicados y remover primitivas degeneradas
		vtkCleanPolyData *cleanFilter = vtkCleanPolyData::New();
        cleanFilter->SetInput ( cubesFilter->GetOutput() );
		
		// crea poligonos triangulares
        vtkTriangleFilter *triangleFilter = vtkTriangleFilter::New();
        triangleFilter->SetInput( cleanFilter->GetOutput() );
        triangleFilter->Update();

		innerSurface->DeepCopy(triangleFilter->GetOutput());


		// ------------------------------------------------------------------------
		//  2. CALCULATING THE OUTER SURFACE
		// ------------------------------------------------------------------------
		vtkImageContinuousDilate3D *dilateFilter = vtkImageContinuousDilate3D ::New();
        dilateFilter->SetInput (volume);
		dilateFilter->SetKernelSize (3, 3, 3);
		gaussFilter->SetInput( dilateFilter->GetOutput());
        triangleFilter->Update();
		
	    outerSurface->DeepCopy(triangleFilter->GetOutput());

		// ------------------------------------------------------------------------
		//  3. CLEANING UP
		// ------------------------------------------------------------------------

		gaussFilter->Delete();
		cubesFilter->Delete();
		cleanFilter->Delete();
	    triangleFilter->Delete();
        dilateFilter->Delete();
}
